Class: Jekyll::Embed
- Inherits:
-
Object
- Object
- Jekyll::Embed
- Defined in:
- lib/jekyll/embed.rb,
lib/jekyll/embed/cache.rb,
lib/jekyll/embed/filter.rb,
lib/jekyll/embed/content.rb
Overview
The idea with this class is to find the best safe representation of a link. For a YouTube video it could be the sandboxed iframe. This loads the video and allows you to reproduce it while preventing YT to call home and send data about your users. But other social networks will try to take control of their containers by modifying the page. They resist sandboxing and don't work correctly. For them, we cleanup unwanted HTML tags such as